package org.geek.szbay.assistant.code.generator;

import java.util.List;

import org.geek.szbay.assistant.code.generator.helpper.MakeHelpper;
import org.geek.szbay.assistant.code.generator.helpper.TableHelpper;
import org.geek.szbay.assistant.utils.StringUtils;

/**
 * @author wutianbin
 * @Time：2017年5月18日 下午7:31:51
 * @version 1.0
 */
public class ControllerMaker {

    /** 
     * 功能：生成实体类主体代码 
     * @param colnames 
     * @param colTypes 
     * @param colSizes 
     * @return 
     */  
    public static final String parse(String authorName, String className, 
            String tableName, String tableComment, String key, String outPath, 
            String entityFullClassName, String entityClassName,
            String serviceFullClassName, String serviceClassName,
            String dtoRootPath, List<String> attrNames, 
            List<String> colNames, List<String> colTypes, List<String> colComments) {  
        StringBuffer sb = new StringBuffer();  
          
        sb.append("package " + outPath + ";\r\n");  
        sb.append("\r\n");  
        
        sb.append("import org.springframework.beans.factory.annotation.Autowired;\r\n");
        sb.append("import org.springframework.stereotype.Controller;\r\n");
        sb.append("import org.springframework.ui.Model;\r\n");
        sb.append("import org.springframework.web.bind.annotation.RequestMapping;\r\n");
        sb.append("import org.springframework.web.bind.annotation.ResponseBody;\r\n");
        sb.append("\r\n");  

        sb.append("import " + entityFullClassName + ";\r\n");
        sb.append("import " + dtoRootPath + ".Page;\r\n");
        sb.append("import " + dtoRootPath + ".RetResult;\r\n");
        sb.append("import " + serviceFullClassName + ";\r\n");
        sb.append("\r\n");  

        //注释部分  
        sb.append(MakeHelpper.makeClassComments(tableComment + "(" + tableName + ") 的后台维护控制类", authorName));
        
        //实体部分  
        sb.append("@Controller\r\n");
        sb.append("public class ").append(className)
        	.append(" extends BaseController<").append(entityClassName).append("> {\r\n");
        sb.append("\r\n");
        
        String serviceObject = StringUtils.firstToLowerCase(serviceClassName);
 
        sb.append("    @Autowired\r\n");
        sb.append("    private ").append(serviceClassName).append(" ").append(serviceObject).append(";\r\n");
        sb.append("\r\n");
        
        String tname = TableHelpper.tableNameToClassName(tableName);
        String tableUrl = "/" + StringUtils.firstToLowerCase(tname) + "/";
        
    	sb.append("    @RequestMapping(\"").append(tableUrl).append("save\")\r\n");
    	sb.append("    @Override\r\n");
    	sb.append("    public RetResult<Integer> save(").append(entityClassName).append(" t) {\r\n");
    	sb.append("        return super.save(t);\r\n");
    	sb.append("    }\r\n");
        sb.append("\r\n");
        
        sb.append("    @RequestMapping(\"").append(tableUrl).append("savePage\")\r\n");
    	sb.append("    @Override\r\n");
        sb.append("    public String savePage(").append(entityClassName).append(" t, Model model) {\r\n");
        sb.append("        return super.savePage(t, model);\r\n");
        sb.append("    }\r\n");
        sb.append("\r\n");

        sb.append("    @RequestMapping(\"").append(tableUrl).append("delete\")\r\n");
    	sb.append("    @Override\r\n");
        sb.append("    public RetResult<Integer> delete(String key) {\r\n");
        sb.append("        return super.delete(key);\r\n");
        sb.append("    }\r\n");
        sb.append("\r\n");
        
        sb.append("    @RequestMapping(\"").append(tableUrl).append("deletePage\")\r\n");
    	sb.append("    @Override\r\n");
        sb.append("    public String deletePage(String key, Model model) {\r\n");
        sb.append("        return super.deletePage(key, model);\r\n");
        sb.append("    }\r\n");
        sb.append("\r\n");

        sb.append("    @RequestMapping(\"").append(tableUrl).append("update\")\r\n");
    	sb.append("    @Override\r\n");
        sb.append("    public RetResult<Integer> update(").append(entityClassName).append(" t) {\r\n");
        sb.append("        return super.update(t);\r\n");
        sb.append("    }\r\n");
        sb.append("\r\n");
        
        sb.append("    @RequestMapping(\"").append(tableUrl).append("updatePage\")\r\n");
    	sb.append("    @Override\r\n");
        sb.append("    public String updatePage(").append(entityClassName).append(" t, Model model) {\r\n");
        sb.append("        return super.updatePage(t, model);\r\n");
        sb.append("    }\r\n");
        sb.append("\r\n");     
        
        sb.append("    @RequestMapping(\"").append(tableUrl).append("getByKey\")\r\n");
    	sb.append("    @Override\r\n");
        sb.append("    public RetResult<").append(entityClassName).append("> getByKey(String key) {\r\n");
        sb.append("        return super.getByKey(key);\r\n");
        sb.append("    }\r\n");
        sb.append("\r\n");     

        sb.append("    @RequestMapping(\"").append(tableUrl).append("getByKeyPage\")\r\n");
    	sb.append("    @Override\r\n");
        sb.append("    public String getByKeyPage(String key, Model model) {\r\n");
        sb.append("        return super.getByKeyPage(key, model);\r\n");
        sb.append("    }\r\n");
        sb.append("\r\n");  
        
        sb.append("    @RequestMapping(\"").append(tableUrl).append("list\")\r\n");
    	sb.append("    @ResponseBody\r\n");
    	sb.append("    @Override\r\n");
        sb.append("    public RetResult<Page<").append(entityClassName).append(">> list(")
        		.append(entityClassName).append(" example) {\r\n");
        sb.append("        return super.list(example);\r\n");
        sb.append("    }\r\n");
        sb.append("\r\n");  
      
        sb.append("    @RequestMapping(\"").append(tableUrl).append("listPage\")\r\n");
    	sb.append("    @Override\r\n");
        sb.append("    public String listPage(Model model, ").append(entityClassName).append(" example) {\r\n");
        sb.append("        return super.listPage(model, example);\r\n");
        sb.append("    }\r\n");
        sb.append("\r\n");     

        sb.append("}\r\n");  

        return sb.toString();  
    }

}
