package org.geek.szbay.assistant.code.generator;

import java.util.List;

import org.geek.szbay.assistant.code.generator.helpper.MakeHelpper;
import org.geek.szbay.assistant.utils.StringUtils;

/**
 * @author wutianbin
 * @Time：2017年5月18日 下午7:31:51
 * @version 1.0
 */
public class EntityMysqlMaker {

    /** 
     * 功能：生成实体类主体代码 
     * @param colnames 
     * @param colTypes 
     * @param colSizes 
     * @return 
     */  
    public static final String parse(String authorName, String className, 
            String tableName, String tableComment, String key, 
            String entityOutPath, List<String> attrNames, 
            List<String> colNames, List<String> colTypes, List<String> colComments) {  
        StringBuffer sb = new StringBuffer();  
          
        sb.append("package " + entityOutPath + ";\r\n");  
        sb.append("\r\n");  
        
        if (haveDateType(colTypes)) {
        	sb.append("import java.util.Date;\r\n");
        	sb.append("\r\n");  
        }
        
        //类注释部分  
        sb.append(MakeHelpper.makeClassComments(tableComment + "(" + tableName + ") 实体类", authorName));

        //实体部分  
        sb.append("public class " + className + " {\r\n");  
        processAllAttrs(sb, attrNames, colNames, colTypes, colComments);//属性  
        sb.append("\r\n");  
        processAllMethod(sb, attrNames, colNames, colTypes, colComments);//get set方法  
        sb.append("\r\n");  
        
        parseHashCode(sb, attrNames);
        parseEquals(sb, className, attrNames);
        parseToString(sb, className, attrNames);

        sb.append("}\r\n"); 
        sb.append("\r\n");  

        return sb.toString();  
    }
    
    private static final boolean haveDateType(List<String> colTypes) {
    	for(String t : colTypes) {
    		String typ = t.toLowerCase(); 
    		 if (typ.indexOf("time") >= 0 || typ.indexOf("date") >= 0) {
    			 return true;
    		 }
    	}
    	return false;
    }
    
    private static final void parseHashCode(StringBuffer sb, List<String> attrNames) {  
        sb.append("    @Override\r\n");
        sb.append("    public int hashCode() { \r\n");
        sb.append("        final int prime = 31;\r\n");
        sb.append("\r\n");
        sb.append("        int resultHashCode = 1;\r\n");
        sb.append("\r\n");
        for(String attr : attrNames) {
            sb.append("        resultHashCode = prime * resultHashCode + ((").append(attr).append(" == null) ? 0 : ").append(attr).append(".hashCode());\r\n");
        }
        sb.append("\r\n");
        sb.append("        return resultHashCode;\r\n");
        sb.append("    }\r\n");
        sb.append("\r\n");
    }
    

    private static final void parseEquals(StringBuffer sb, String className, List<String> attrNames) {  
        sb.append("    @Override\r\n");
        sb.append("    public boolean equals(Object obj) { \r\n");
        sb.append("        if (this == obj) {\r\n");
        sb.append("            return true;\r\n");
        sb.append("        }\r\n");
        sb.append("        if (obj == null) {\r\n");
        sb.append("            return false;\r\n");
        sb.append("        }\r\n");
        sb.append("        if (getClass() != obj.getClass()) {\r\n");
        sb.append("            return false;\r\n");
        sb.append("        }\r\n");
        sb.append("\r\n");
        sb.append("        ").append(className).append(" other = (").append(className).append(") obj;\r\n");
        sb.append("\r\n");
        
        for(String attr : attrNames) {
            sb.append("        if (").append(attr).append(" == null) {\r\n");
            sb.append("            if (other.").append(attr).append(" != null) {\r\n");
            sb.append("                return false;\r\n");
            sb.append("            }\r\n");
            sb.append("        } else if (!").append(attr).append(".equals(other.").append(attr).append(")) {\r\n");
            sb.append("            return false; \r\n");
            sb.append("        }\r\n");        
            sb.append("\r\n");
        }
        
        sb.append("        return true;\r\n");

        sb.append("    }\r\n");
        sb.append("\r\n");
    }

    private static final void parseToString(StringBuffer sb, String className, List<String> attrNames) {  
        sb.append("    @Override\r\n");
        sb.append("    public String toString() { \r\n");
        sb.append("        StringBuffer sb = new StringBuffer();\r\n");
        sb.append("\r\n");
        sb.append("        sb.append(\"").append(className).append(" [\");\r\n");
        
        for(int i = 0; i < attrNames.size(); i++) {
            String attr = attrNames.get(i);
            
            sb.append("        sb.append(\"").append(attr).append("=\"").append(").append(").append(attr);
            if (i < attrNames.size() - 1) {
                sb.append(").append(\",\");\r\n");    
            } else {
                sb.append(");\r\n");    
            }
        }
        sb.append("        sb.append(\"]\");\r\n");
        sb.append("\r\n");
        sb.append("        return sb.toString();\r\n");
        sb.append("    }\r\n");
        sb.append("\r\n");
    }

    /** 
     * 功能：生成所有属性 
     * @param sb 
     */  
    private static final void processAllAttrs(StringBuffer sb, List<String> attrNames, List<String> colNames, 
            List<String> colTypes, List<String> colComments) {  
          
        for (int i = 0; i < colNames.size(); i++) {  
            sb.append("    /** " + colComments.get(i) + " */;\r\n");
            sb.append("    private " + colTypes.get(i) + " " + attrNames.get(i) + ";\r\n");
            sb.append("\r\n");
        }  
    }  
  
    /** 
     * 功能：生成所有方法 
     * @param sb 
     */  
    private static final void processAllMethod(StringBuffer sb, List<String> attrNames, List<String> colNames, 
            List<String> colTypes, List<String> colComments) {  
        String attr = "";
        String comment = "";
        String type = "";
        String attrMethod = "";
        
        for (int i = 0; i < attrNames.size(); i++) {  
            attr = attrNames.get(i);
            comment = colComments.get(i);
            type = colTypes.get(i);
            attrMethod = StringUtils.firstToUpperCase(attr);
            
            sb.append("    /** setter: " + comment + " */;\r\n");
            sb.append("    public void set" + attrMethod + "(" + type + " " +   
                    attrNames.get(i) + ") {\r\n");  
            sb.append("        this." + attr + " = " + attr + ";\r\n");  
            sb.append("    }\r\n\r\n");  
            sb.append("    /** getter: " + comment + " */;\r\n");
            sb.append("    public " + type + " get" + attrMethod + "() {\r\n");  
            sb.append("        return " + attr + ";\r\n");  
            sb.append("    }\r\n");  
            sb.append("\r\n");
        }  
          
    }  

}
