package org.geek.szbay.assistant.code.generator;

import java.util.List;

import org.geek.szbay.assistant.code.generator.helpper.TableHelpper;
import org.geek.szbay.assistant.utils.StringUtils;

/**
<?xml version="1.0" encoding="UTF-8" ?>
<!DOCTYPE mapper PUBLIC "-//mybatis.org//DTD Mapper 3.0//EN" "http://mybatis.org/dtd/mybatis-3-mapper.dtd">
<mapper namespace="com.travel.pay.dao.write.pay.PayOrderLogReadDao">

    <resultMap id="payOrderLogMap" type="com.travel.pay.model.pay.PayOrderLogDO">
        <id column="id" property="id"/>
        <result column="pay_order_id" property="payOrderId"/>
        <result column="type" property="type"/>
        <result column="result" property="result"/>
        <result column="input" property="input"/>
        <result column="output" property="output"/>
        <result column="gmt_create" property="gmtCreate"/>
        <result column="gmt_modified" property="gmtModified"/>
    </resultMap>

    <sql id="allField">
        id,pay_order_id,type,result,input,output,gmt_create,gmt_modified
    </sql>

    <select id="getById" parameterType="java.lang.Long" resultMap="payOrderLogMap">
        select <include refid="allField"/>
        from pay_order_log where id = #{id}
    </select>

</mapper>
 * @author wutianbin
 * @Time：2017年5月18日 下午7:31:51
 * @version 1.0
 */
public class ReadMapperMaker {

    public static final String parse(String authorName, String tableName, String key, String outPath, 
            String entityFullClassName, String entityClassName,
            String readFullClassName, String readClassName,
            List<String> attrNames, List<String> colNames, List<String> colTypes, List<String> colComments) {  
        StringBuffer sb = new StringBuffer();  
        String className = TableHelpper.columnNameToAttrName(tableName); 
        String keyAttr = TableHelpper.columnNameToAttrName(key);
        
        parseHead(sb);
        parseNamespace(sb, readFullClassName);
        parseResultMap(sb, key, keyAttr, className, entityFullClassName, colNames, attrNames);
        parseAllField(sb, colNames);
        parseGetByKey(sb, key, keyAttr, className, tableName);
        parseEnd(sb);
        
        return sb.toString();
    }
    
    private static final void parseHead(StringBuffer sb) {  
        sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>").append("\r\n");  
        sb.append("<!DOCTYPE mapper PUBLIC \"-//mybatis.org//DTD Mapper 3.0//EN\" \"http://mybatis.org/dtd/mybatis-3-mapper.dtd\">").append("\r\n");
    }

    private static final void parseNamespace(StringBuffer sb, String readFullClassName) {  
        sb.append("<mapper namespace=\"").append(readFullClassName).append("\"> ").append("\r\n");  
        sb.append("\r\n");  
    }

    private static final void parseResultMap(StringBuffer sb, String key, String keyAttr, String className,  
            String entityFullClassName, List<String> colNames, List<String> attrNames) {
        sb.append("    <resultMap ").append("id=\"").append(className).append("Map\" type=\"").append(entityFullClassName).append("\">").append("\r\n");
        sb.append("        <id column=\"").append(key).append("\" property=\"").append(keyAttr).append("\"/>").append("\r\n");
        
        for(int i = 0; i < colNames.size(); i++) {
            String column = colNames.get(i); 
            if (column.equals(key)) {
                continue;
            }
            
            String attr = attrNames.get(i);
            
            sb.append("        <result column=\"").append(column).append("\" property=\"").append(attr).append("\"/>").append("\r\n");
        }
        
        sb.append("    </resultMap>\r\n");
        sb.append("\r\n");
    }

    private static final void parseAllField(StringBuffer sb, List<String> colNames) {  
        sb.append("    <sql id=\"allField\">").append("\r\n");
        sb.append("        ");
        
        for(int i = 0; i < colNames.size(); i++) {
            String column = colNames.get(i); 
            sb.append(column);
            if (i != colNames.size() - 1) {
                sb.append(",");
            }
        }
        
        sb.append("\r\n");
        sb.append("    </sql>\r\n");
        sb.append("\r\n");
    }

    private static final void parseGetByKey(StringBuffer sb, String key, String keyAttr, String className, String tableName) {  
        sb.append("    <select id=\"getBy").append(StringUtils.firstToUpperCase(keyAttr)).append("\" ")
                .append("parameterType=\"java.lang.Long\" resultMap=\"")
                .append(StringUtils.firstToLowerCase(className)).append("Map\">").append("\r\n");
        sb.append("        select <include refid=\"allField\"/>\r\n");
        sb.append("        from ").append(tableName).append(" where ").append(key).append(" = #{").append(keyAttr).append("}\r\n");
        sb.append("    </select>");
        sb.append("\r\n");
    }

    private static final void parseEnd(StringBuffer sb) {  
        sb.append("\r\n");
        sb.append("</mapper>");
        sb.append("\r\n");
    }

}
