package org.geek.szbay.assistant.code.generator;

import java.util.List;

import org.geek.szbay.assistant.code.generator.helpper.TableHelpper;

/**
<?xml version="1.0" encoding="UTF-8" ?>
<!DOCTYPE mapper PUBLIC "-//mybatis.org//DTD Mapper 3.0//EN" "http://mybatis.org/dtd/mybatis-3-mapper.dtd">
<mapper namespace="com.travel.pay.dao.write.pay.PayOrderLogWriteDao">

    <insert id="save" useGeneratedKeys="true" keyProperty="id" parameterType="com.travel.pay.model.pay.PayOrderLogDO">
        insert into pay_order_log
        <trim prefix="(" suffix=")" suffixOverrides=",">
            <if test="payOrderId != null" >
                pay_order_id,
            </if>
            <if test="type != null" >
                type,
            </if>
            <if test="result != null" >
                result,
            </if>
            <if test="input != null" >
                input,
            </if>
            <if test="output != null" >
                output,
            </if>
        </trim>
        <trim prefix="values (" suffix=")" suffixOverrides=",">
            <if test="payOrderId != null" >
                #{payOrderId},
            </if>
            <if test="type != null" >
                #{type},
            </if>
            <if test="result != null" >
                #{result},
            </if>
            <if test="input != null" >
                #{input},
            </if>
            <if test="output != null" >
                #{output},
            </if>
        </trim>
    </insert>

    <update id="update" parameterType="com.travel.pay.model.pay.PayOrderLogDO">
        update pay_order_log
        <set>
            <if test="payOrderId != null" >
                pay_order_id = #{payOrderId},
            </if>
            <if test="type != null" >
                type = #{type},
            </if>
            <if test="result != null" >
                result = #{result},
            </if>
            <if test="input != null" >
                input = #{input},
            </if>
            <if test="output != null" >
                output = #{output},
            </if>
        </set>
        where id = #{id}
    </update>

</mapper>
 * @author wutianbin
 * @Time：2017年5月18日 下午7:31:51
 * @version 1.0
 */
public class WriteMapperMaker {

    public static final String parse(String authorName,  
            String tableName, String key, String outPath, 
            String entityFullClassName, String entityClassName,
            String writeFullClassName, String writeClassName,
            List<String> attrNames, 
            List<String> colNames, List<String> colTypes, List<String> colComments) {  
        StringBuffer sb = new StringBuffer();  
        String keyAttr = TableHelpper.columnNameToAttrName(key);
        
        parseHead(sb);
        parseNamespace(sb, writeFullClassName);
        
        parseInsert(sb, key, keyAttr, tableName, entityFullClassName, colNames, attrNames);
        
        parseUpdate(sb, key, keyAttr, tableName, entityFullClassName, colNames, attrNames);
        
        parseEnd(sb);
        
        return sb.toString();
    }
    
    private static final void parseHead(StringBuffer sb) {  
        sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>").append("\r\n");  
        sb.append("<!DOCTYPE mapper PUBLIC \"-//mybatis.org//DTD Mapper 3.0//EN\" \"http://mybatis.org/dtd/mybatis-3-mapper.dtd\">").append("\r\n");
    }

    private static final void parseNamespace(StringBuffer sb, String writeFullClassName) {  
        sb.append("<mapper namespace=\"").append(writeFullClassName).append("\"> ").append("\r\n");  
        sb.append("\r\n");  
    }

    private static final void parseInsert(StringBuffer sb, String key, String keyAttr, String tableName,  
            String entityFullClassName, List<String> colNames, List<String> attrNames) {
        sb.append("    <insert id=\"save\" useGeneratedKeys=\"");
        if ("id".equals(key)) {
            sb.append("true\" ");    
        } else {
            sb.append("false\" ");
        }
        sb.append("keyProperty=\"").append(keyAttr).append("\" ");
        sb.append("parameterType=\"").append(entityFullClassName).append("\">\r\n");
        
        sb.append("        insert into ").append(tableName).append("\r\n");
        sb.append("        <trim prefix=\"(\" suffix=\")\" suffixOverrides=\",\">\r\n");

        for(int i = 0; i < colNames.size(); i++) {
            String column = colNames.get(i);
            String attr = attrNames.get(i);
            
            if (column.equals(key) && "id".equals(key)) {
                // key自动生成，所以不做任何处理
                continue;
            }
            
            sb.append("            <if test=\"").append(attr).append(" != null\" >\r\n");
            sb.append("                ").append(column).append(", \r\n");
            sb.append("            </if>\r\n");
        }
        
        sb.append("        </trim>\r\n");
        
        sb.append("        <trim prefix=\"values (\" suffix=\")\" suffixOverrides=\",\">\r\n");
        
        for(int i = 0; i < colNames.size(); i++) {
            String attr = attrNames.get(i); 
            if (attr.equals("id")) {
                continue;
            }

            sb.append("            <if test=\"").append(attr).append(" != null\" >\r\n");
            sb.append("                #{").append(attr).append("}, \r\n");
            sb.append("            </if>\r\n");
        }

        sb.append("        </trim>\r\n");

        sb.append("    </insert>\r\n");
        sb.append("\r\n");
    }

    private static final void parseUpdate(StringBuffer sb, String key, String keyAttr, String tableName, 
            String entityFullClassName, List<String> colNames, List<String> attrNames) {  
        sb.append("    <update id=\"update\"");
        sb.append(" parameterType=\"").append(entityFullClassName).append("\">\r\n");
        
        sb.append("        update ").append(tableName).append("\r\n");
        sb.append("        <set>\r\n");

        for(int i = 0; i < colNames.size(); i++) {
            String column = colNames.get(i);
            String attr = attrNames.get(i);
            
            if (column.equals(key)) {
                // key不做任何处理
                continue;
            }
            
            sb.append("            <if test=\"").append(attr).append(" != null\" >\r\n");
            sb.append("                ").append(column).append(" = #{").append(attr).append("}, \r\n");
            sb.append("            </if>\r\n");
        }
        
        sb.append("        </set>\r\n");
        
        sb.append("        where ").append(key).append(" = #{").append(keyAttr).append("}\r\n");
        
        sb.append("    </update>");
        sb.append("\r\n");
    }

    private static final void parseEnd(StringBuffer sb) {  
        sb.append("\r\n");
        sb.append("</mapper>");
        sb.append("\r\n");
    }

}
